
def get_model_name(klass):
    if hasattr(klass._meta, 'model_name'):
        model_name = klass._meta.model_name
    else:  # Django < 1.6
        model_name = klass._meta.module_name

    return model_name



import pytest

from .models import TestModel, TestModelWithSpecifiedFields, TestModelWithM2MAndSpecifiedFields, TestModelWithSpecifiedFieldsAndForeignKey


@pytest.mark.django_db
def test_dirty_fields_on_model_with_specified_fields():
    tm = TestModelWithSpecifiedFields.objects.create()

    tm.boolean1 = False
    tm.boolean2 = False

    # boolean1 is tracked, boolean2 isn`t tracked
    assert tm.get_dirty_fields() == {'boolean1': True}


@pytest.mark.django_db
def test_dirty_fields_on_model_with_m2m_and_specified_fields():
    tm = TestModelWithM2MAndSpecifiedFields.objects.create()
    tm2 = TestModel.objects.create()

    tm.m2m1.add(tm2)
    tm.m2m2.add(tm2)

    # m2m1 is tracked, m2m2 isn`t tracked
    assert tm.get_dirty_fields(check_m2m={'m2m1': set([])}) == {'m2m1': set([tm2.id])}
    assert tm.get_dirty_fields(check_m2m={'m2m2': set([])}) == {}


@pytest.mark.django_db
def test_dirty_fields_on_model_with_specified_fields_can_save_when_non_tracked_field_is_modified():
    tm = TestModelWithSpecifiedFields.objects.create()

    tm.boolean1 = False
    tm.boolean2 = False

    tm.save(update_fields=["boolean2"])

    assert "boolean1" in tm._original_state
    assert "boolean2" not in tm._original_state


@pytest.mark.django_db
def test_dirty_fields_on_model_with_specified_fields_can_save_when_non_tracked_fk_field_is_modified():
    tm = TestModelWithSpecifiedFieldsAndForeignKey.objects.create()
    fk = TestModel.objects.create()
    tm.fk_field = fk
    tm.boolean1 = False
    tm.boolean2 = False

    tm.save(update_fields=["fk_field"])
    assert "fk_field" in tm._original_state

    tm.boolean2 = True
    tm.save()
    assert "fk_field" in tm._original_state
    assert "boolean2" not in tm._original_state



import sys
import django


def get_m2m_with_model(given_model):
    if django.VERSION < (1, 9):
        return given_model._meta.get_m2m_with_model()
    else:
        return [
            (f, f.model if f.model != given_model else None)
            for f in given_model._meta.get_fields()
            if f.many_to_many and not f.auto_created
        ]


def is_buffer(value):
    if sys.version_info < (3, 0, 0):
        return isinstance(value, buffer)  # noqa
    else:
        return isinstance(value, memoryview)


def remote_field(field):
    if django.VERSION < (1, 9):
        return field.rel
    else:
        return field.remote_field



import resource

import pytest

from .models import TestModel as DirtyMixinModel

pytestmark = pytest.mark.django_db


def test_rss_usage():
    DirtyMixinModel()
    rss_1 = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    for _ in range(1000):
        DirtyMixinModel()
    rss_2 = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    assert rss_2 == rss_1, 'There is a memory leak!'



import unittest
import pytest
from django.db import models
from dirtyfields import DirtyFieldsMixin

JSON_FIELD_AVAILABLE = False
try:
    from jsonfield import JSONField
    JSON_FIELD_AVAILABLE = True
except ImportError:
    pass


if JSON_FIELD_AVAILABLE:
    class JSONFieldModel(DirtyFieldsMixin, models.Model):
        json_field = JSONField()


@unittest.skipIf(not JSON_FIELD_AVAILABLE, 'django jsonfield library required')
@pytest.mark.django_db
def test_json_field():
    tm = JSONFieldModel.objects.create(json_field={'data': [1, 2, 3]})

    data = tm.json_field['data']
    data.append(4)

    assert tm.get_dirty_fields() == {
        'json_field': {'data': [1, 2, 3]}
    }



import django
import pytest

from .models import TestModel, TestMixedFieldsModel, TestModelWithForeignKey
from .utils import assert_number_of_queries_on_regex


@pytest.mark.django_db
def test_save_dirty_simple_field():
    tm = TestModel.objects.create()

    tm.characters = 'new_character'

    with assert_number_of_queries_on_regex(r'.*characters.*', 1):
        with assert_number_of_queries_on_regex(r'.*boolean.*', 1):
            tm.save()

    tm.characters = 'new_character_2'

    # Naive checking on fields involved in Django query
    # boolean unchanged field is not updated on Django update query: GOOD !
    with assert_number_of_queries_on_regex(r'.*characters.*', 1):
        with assert_number_of_queries_on_regex(r'.*boolean.*', 0):
            tm.save_dirty_fields()

    # We also check that the value has been correctly updated by our custom function
    assert tm.get_dirty_fields() == {}
    assert TestModel.objects.get(pk=tm.pk).characters == 'new_character_2'


@pytest.mark.django_db
def test_save_dirty_related_field():
    tm1 = TestModel.objects.create()
    tm2 = TestModel.objects.create()
    tmfm = TestMixedFieldsModel.objects.create(fkey=tm1)

    tmfm.fkey = tm2

    with assert_number_of_queries_on_regex(r'.*fkey_id.*', 1):
        with assert_number_of_queries_on_regex(r'.*characters.*', 1):
            tmfm.save()

    tmfm.fkey = tm1

    # Naive checking on fields involved in Django query
    # characters unchanged field is not updated on Django update query: GOOD !
    with assert_number_of_queries_on_regex(r'.*fkey_id.*', 1):
        with assert_number_of_queries_on_regex(r'.*characters.*', 0):
            tmfm.save_dirty_fields()

    # We also check that the value has been correctly updated by our custom function
    assert tmfm.get_dirty_fields() == {}
    assert TestMixedFieldsModel.objects.get(pk=tmfm.pk).fkey_id == tm1.id


@pytest.mark.django_db
def test_save_only_specific_fields_should_let_other_fields_dirty():
    tm = TestModel.objects.create(boolean=True, characters='dummy')

    tm.boolean = False
    tm.characters = 'new_dummy'

    tm.save(update_fields=['boolean'])

    # 'characters' field should still be dirty, update_fields was only saving the 'boolean' field in the db
    assert tm.get_dirty_fields() == {'characters': 'dummy'}


@pytest.mark.django_db
def test_handle_foreignkeys_id_field_in_update_fields():
    tm1 = TestModel.objects.create(boolean=True, characters='dummy')
    tm2 = TestModel.objects.create(boolean=True, characters='dummy')
    tmwfk = TestModelWithForeignKey.objects.create(fkey=tm1)

    tmwfk.fkey = tm2
    assert tmwfk.get_dirty_fields(check_relationship=True) == {'fkey': tm1.pk}

    tmwfk.save(update_fields=['fkey_id'])
    assert tmwfk.get_dirty_fields(check_relationship=True) == {}


@pytest.mark.django_db
def test_correctly_handle_foreignkeys_id_field_in_update_fields():
    tm1 = TestModel.objects.create(boolean=True, characters='dummy')
    tm2 = TestModel.objects.create(boolean=True, characters='dummy')
    tmwfk = TestModelWithForeignKey.objects.create(fkey=tm1)

    tmwfk.fkey_id = tm2.pk
    assert tmwfk.get_dirty_fields(check_relationship=True) == {'fkey': tm1.pk}

    tmwfk.save(update_fields=['fkey'])
    assert tmwfk.get_dirty_fields(check_relationship=True) == {}


@pytest.mark.django_db
def test_save_deferred_field_with_update_fields():
    TestModel.objects.create()

    tm = TestModel.objects.defer('boolean').first()
    tm.boolean = False
    # Test that providing a deferred field to the update_fields
    # save parameter doesn't raise a KeyError anymore.
    tm.save(update_fields=['boolean'])


@pytest.mark.django_db
def test_deferred_field_was_not_dirty():
    TestModel.objects.create()
    tm = TestModel.objects.defer('boolean').first()
    tm.boolean = False
    assert tm.get_dirty_fields() == {}


@pytest.mark.django_db
def test_save_deferred_field_with_update_fields_behaviour():
    """ Behaviour of deferred fields has changed in Django 1.10.
    Once explicitly updated (using the save update_fields parameter),
    a field cannot be considered deferred anymore.
    """

    TestModel.objects.create()
    tm = TestModel.objects.defer('boolean').first()
    tm.save(update_fields=['boolean'])
    tm.boolean = False
    if django.VERSION < (1, 10):
        assert tm.get_dirty_fields() == {}
    else:
        assert tm.get_dirty_fields() == {'boolean': True}



import pytest

from tests.utils import is_postgresql_env_with_json_field


@pytest.mark.skipif(not is_postgresql_env_with_json_field(),
                    reason="requires postgresql and Django 1.9+")
@pytest.mark.django_db
def test_dirty_json_field():
    from tests.models import TestModelWithJSONField

    tm = TestModelWithJSONField.objects.create(json_field={'data': [1, 2, 3]})

    data = tm.json_field['data']
    data.append(4)

    assert tm.get_dirty_fields(verbose=True) == {
        'json_field': {
            'current': {'data': [1, 2, 3, 4]},
            'saved': {'data': [1, 2, 3]}
        }
    }



import datetime
import pytz
import warnings

from django.utils import timezone


def compare_states(new_state, original_state, compare_function):
    modified_field = {}

    for key, value in new_state.items():
        try:
            original_value = original_state[key]
        except KeyError:
            # In some situation, like deferred fields, it can happen that we try to compare the current
            # state that has some fields not present in original state because of being initially deferred.
            # We should not include them in the comparison.
            continue

        is_identical = compare_function[0](value, original_value, **compare_function[1])
        if is_identical:
            continue

        modified_field[key] = {'saved': original_value, 'current': value}

    return modified_field


def raw_compare(new_value, old_value):
    return new_value == old_value


def timezone_support_compare(new_value, old_value, timezone_to_set=pytz.UTC):

    if not (isinstance(new_value, datetime.datetime) and isinstance(old_value, datetime.datetime)):
        return raw_compare(new_value, old_value)

    db_value_is_aware = timezone.is_aware(old_value)
    in_memory_value_is_aware = timezone.is_aware(new_value)

    if db_value_is_aware == in_memory_value_is_aware:
        return raw_compare(new_value, old_value)

    if db_value_is_aware:
        # If db value is aware, it means that settings.USE_TZ=True, so we need to convert in-memory one
        warnings.warn(u"DateTimeField received a naive datetime (%s)"
                      u" while time zone support is active." % new_value,
                      RuntimeWarning)
        new_value = timezone.make_aware(new_value, timezone_to_set).astimezone(pytz.utc)
    else:
        # The db is not timezone aware, but the value we are passing for comparison is aware.
        warnings.warn(u"Time zone support is not active (settings.USE_TZ=False), "
                      u"and you pass a time zone aware value (%s)"
                      u" Converting database value before comparison." % new_value,
                      RuntimeWarning)
        old_value = timezone.make_aware(old_value, pytz.utc).astimezone(timezone_to_set)

    return raw_compare(new_value, old_value)
